"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const fast_deep_equal_1 = __importDefault(require("fast-deep-equal"));
function registerTaskDef(params) {
    return new Promise((resolve, reject) => {
        const ecs = new aws_sdk_1.default.ECS({ apiVersion: '2014-11-13', region: 'eu-west-1' });
        ecs.registerTaskDefinition(params, (err, res) => {
            if (err) {
                console.log('Error writing Task Definition', JSON.stringify(params));
                reject(err);
                return;
            }
            resolve(res);
        });
    });
}
exports.registerTaskDef = registerTaskDef;
// https://docs.aws.amazon.com/AWSJavaScriptSDK/latest/AWS/ECS.html#describeTaskDefinition-property
function describeTaskDef(taskDef) {
    return new Promise((resolve, reject) => {
        const ecs = new aws_sdk_1.default.ECS({ apiVersion: '2014-11-13', region: 'eu-west-1' });
        const params = {
            taskDefinition: taskDef,
        };
        ecs.describeTaskDefinition(params, (err, res) => {
            if (err) {
                reject(err);
            }
            else {
                resolve(res);
            }
        });
    });
}
exports.describeTaskDef = describeTaskDef;
function orderArray(input, key) {
    return input.sort((a, b) => {
        if (a[key] > b[key])
            return 1;
        if (a[key] < b[key])
            return -1;
        return 0;
    });
}
// I've added any because the types do not match the payl
// false = they are not the same
// true = they are the same
function compareDefs(old, recent) {
    var _a, _b, _c, _d;
    const server = old.taskDefinition;
    const local = recent;
    const localContainer = local.containerDefinitions[0];
    const serverContainer = server.containerDefinitions[0];
    if (((_a = server) === null || _a === void 0 ? void 0 : _a.cpu) !== local.cpu)
        return false;
    if (((_b = server) === null || _b === void 0 ? void 0 : _b.memory) !== local.memory)
        return false;
    if (((_c = localContainer) === null || _c === void 0 ? void 0 : _c.image) !== serverContainer.image)
        return false;
    if (((_d = localContainer) === null || _d === void 0 ? void 0 : _d.name) !== serverContainer.name)
        return false;
    const orderedLocalEnv = orderArray(local.containerDefinitions[0].environment, 'name');
    const orderedLocalSec = orderArray(local.containerDefinitions[0].secrets, 'name');
    const orderedServerEnv = orderArray(server.containerDefinitions[0].environment, 'name');
    const orderedServerSec = orderArray(server.containerDefinitions[0].secrets, 'name');
    const environment = fast_deep_equal_1.default(orderedLocalEnv, orderedServerEnv);
    const secrets = fast_deep_equal_1.default(orderedLocalSec, orderedServerSec);
    if (environment === false || secrets === false)
        return false;
    return true;
}
exports.compareDefs = compareDefs;
function createTaskDefFromStack(tranformation, opts, name, env) {
    const params = {
        family: tranformation.taskFamily,
        executionRoleArn: `arn:aws:iam::${opts[env].aws}:role/ecsTaskExecutionRole`,
        containerDefinitions: [{
                logConfiguration: {
                    logDriver: 'awslogs',
                    options: {
                        'awslogs-group': `/ecs/${tranformation.clusterName}`,
                        'awslogs-region': opts[env].region,
                        'awslogs-stream-prefix': 'ecs',
                    },
                },
                portMappings: [],
                cpu: 0,
                environment: [],
                mountPoints: [],
                secrets: [],
                volumesFrom: [],
                image: tranformation.imageName,
                essential: true,
                name: `${tranformation.clusterName}-${name}`,
            }],
        placementConstraints: [],
        memory: String(opts[env].memory),
        cpu: String(opts[env].cpu),
        taskRoleArn: `arn:aws:iam::${opts[env].aws}:role/ecsTaskExecutionRole`,
        requiresCompatibilities: ['FARGATE'],
        networkMode: 'awsvpc',
        volumes: [],
    };
    opts[env].env.forEach((e) => {
        if (e.secret === true && params.containerDefinitions[0].secrets) {
            params.containerDefinitions[0].secrets.push({
                name: e.key,
                valueFrom: e.value,
            });
        }
        else if (params.containerDefinitions[0].environment) {
            params.containerDefinitions[0].environment.push({
                name: e.key,
                value: e.value,
            });
        }
    });
    return params;
}
exports.createTaskDefFromStack = createTaskDefFromStack;
async function taskDefinition(opts, name, env) {
    const tranformation = {
        clusterName: `${opts.stack}-${opts.type}-${env}`,
        imageName: `${opts[env].aws}.dkr.ecr.${opts[env].region}.amazonaws.com/${opts.stack}-${opts.type}/${name}`,
        taskFamily: `${opts.stack}-${opts.type}-${env}-${name}`,
    };
    const taskDef = createTaskDefFromStack(tranformation, opts, name, env);
    let oldTaskDef;
    try {
        oldTaskDef = await describeTaskDef(tranformation.taskFamily);
    }
    catch (e) {
        console.log('No previous version of the task def found');
    }
    if (!oldTaskDef) {
        console.log('Previous version of task def not found, registering a new one,');
        await registerTaskDef(taskDef);
        return;
    }
    // there was a match, so lets compare and see if there is any need to update the task def
    const defsMatch = compareDefs(oldTaskDef, taskDef);
    if (defsMatch === false) {
        console.log('Task defs do not match, updating');
        await registerTaskDef(taskDef);
        console.log('Task defs has been updated');
        return;
    }
    console.log('No change detected in Task defs');
}
exports.default = taskDefinition;
//# sourceMappingURL=createTaskDefinition.js.map